import sys
import ujson

REPLY_SUCCESS_200 = 200

REPLY_BAD_REQUEST_400 = 400
REPLY_NOT_FOUND_404 = 404
REPLY_METHOD_NOT_ALLOWED_405 = 405
REPLY_CONFLICT_409 = 409

REPLY_INTERNAL_SERVER_ERROR_500 = 500
REPLY_SERVICE_UNAVAILABLE_503 = 503
REPLY_GATEWAY_TIMEOUT = 504

FAIL_CODE_VIDEOSOURCE_ERROR = 'videosource-error'
FAIL_TEXT_VIDEOSOURCE_ERROR = 'something wrong with videosource'


__all__ = [
    'Reply',
    "reply_done",
    "reply_fail",
    "reply_fail2",
    "REPLY_SUCCESS_200",
    "REPLY_BAD_REQUEST_400",
    "REPLY_NOT_FOUND_404",
    "REPLY_METHOD_NOT_ALLOWED_405",
    "REPLY_CONFLICT_409",
    "REPLY_INTERNAL_SERVER_ERROR_500",
    "REPLY_SERVICE_UNAVAILABLE_503",
    "REPLY_GATEWAY_TIMEOUT",
    "FAIL_CODE_VIDEOSOURCE_ERROR",
    "FAIL_TEXT_VIDEOSOURCE_ERROR"
]


class Reply:
    def __init__(self, status=REPLY_SUCCESS_200, body=None):
        """
        :type status: int
        :type body: dict | list | None
        """
        self.status = str(status)
        self.body = body

    def send(self):
        sys.stderr.write(self.status)
        if self.body:
            body_utf8 = ujson.dumps(self.body, ensure_ascii=False)
            sys.stdout.write(body_utf8)


def reply_done(result_jnode=None, envelope=True):
    jnode = {
        'status': REPLY_SUCCESS_200,
        'result': result_jnode if result_jnode else [],
        'envelope': envelope
    }

    return jnode


def reply_fail(http_code, envelope=True):
    """
    :type http_code: int
    :type envelope bool
    :rtype: dict
    """
    jnode = {
        'status': http_code,
        'errors': [],
        'envelope': envelope
    }

    return jnode


def reply_fail2(http_code, error_code, error_text):
    """
    :type http_code: int
    :type error_code: str
    :type error_text: str
    :rtype: dict
    """
    jnode = {
        'status': http_code,
        'errors': [
            {
                'code': error_code,
                'message': error_text
            }
        ]
    }

    return jnode
