import sys
import os
import fcntl
from edge import eventlog

UI_PIPELINE_PID_INDEX = 0
UI_PIPELINE_PREFIX_INDEX = 1
UI_PIPELINE_TYPE_INDEX = 2
UI_PIPELINE_ID_INDEX = 3
UI_PIPELINE_ACTION_INDEX = 4


class BaseCreator:
    def __init__(self):
        pass

    # noinspection PyMethodMayBeStatic
    def serialize(self):
        raise NotImplemented


class LockError(Exception):
    def __init__(self, *args, **kwargs):
        pass


class UiQueryArgs:
    def __init__(self, http_method, path_info, query_string, raw_body):
        """
        :type http_method: str
        :type path_info: list of str
        :type query_string: str | None
        :type raw_body: str | none
        """
        self.http_method = http_method
        self.path_info = path_info
        self.query_string = query_string
        self.raw_body = raw_body

    @staticmethod
    def _read_raw_body():
        """
        :rtype: str | None
        """
        # make stdin a non-blocking file
        fd = sys.stdin.fileno()
        old_flags = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, old_flags | os.O_NONBLOCK)

        try:
            raw_body = sys.stdin.readline()
            if raw_body:
                return raw_body

        except Exception as x:
            eventlog.log_d("fail: read raw-body (%s)" % x.message)

        finally:
            fcntl.fcntl(fd, fcntl.F_SETFL, old_flags)

        return None

    @staticmethod
    def deserialize():
        """
        :rtype: UiQueryArgs
        """
        try:
            http_method = os.environ['REQUEST_METHOD']

            raw_path_info = os.environ['PATH_INFO']
            if raw_path_info:
                path_info = raw_path_info.split('/')[1:]
            else:
                path_info = []

            query_string = os.environ['QUERY_STRING']
            if not query_string:
                query_string = None

            raw_body = UiQueryArgs._read_raw_body()
            eventlog.log_d("%s|%s|%s|%s" % (http_method, raw_path_info, query_string, raw_body))
            query_args = UiQueryArgs(http_method, path_info, query_string, raw_body)
            return query_args

        except Exception as x:
            eventlog.log_d("fail: deserialize query-args (%s)" % x.message)
            raise

    def get_path_info_item(self, index, *args, **kwargs):
        """
        :param index:
        :param args:
        :param kwargs:
        :return:
        """
        if self.path_info and len(self.path_info) > index:
            return self.path_info[index]

        if not (self.path_info and len(self.path_info) > index):
            if len(args) > 0:
                return args[0]

            elif 'def_value' in kwargs:
                return kwargs['def_value']

            else:
                raise IndexError

    def get_ui_pid(self):
        """
        :rtype: str | None
        """
        return self.path_info[UI_PIPELINE_PID_INDEX] if \
            self.path_info and len(self.path_info) > UI_PIPELINE_PID_INDEX else None

    def get_ui_type(self):
        """
        :rtype: str | None
        """
        return self.path_info[UI_PIPELINE_TYPE_INDEX] \
            if self.path_info and len(self.path_info) > UI_PIPELINE_TYPE_INDEX else None

    def get_ui_id(self):
        """
        :rtype: str | None
        """
        return self.path_info[UI_PIPELINE_ID_INDEX] if \
            self.path_info and len(self.path_info) > UI_PIPELINE_ID_INDEX else None

    def get_ui_action(self):
        """
        :rtype: str | None
        """
        return self.path_info[UI_PIPELINE_ACTION_INDEX] if \
            self.path_info and len(self.path_info) > UI_PIPELINE_ACTION_INDEX else None

