from .types import AboutServiceArgs
from ..com import fs
from ..model.types import Origin, Location
from ..service.gpsd import IGpsdService


class AboutError(RuntimeError):
    pass


class IAboutService:
    def __init__(self):
        pass

    def get_origin(self, ts):
        """
        :type ts: long
        :rtype: Origin
        """
        raise NotImplementedError

    def get_hw_serial(self):
        """
        :rtype: str
        """
        raise NotImplementedError


class _Service:
    def __init__(self, args, gpsd_service):
        """
        :type args: AboutServiceArgs
        :type gpsd_service: IGpsdService
        """
        self._gpsd = gpsd_service

        def get_hw_serial():
            jnode = fs.load_json(args.server_about_path) or dict()
            hw_jnode = jnode.get('hw') or list()
            return hw_jnode[2] if len(hw_jnode) == 3 else ''

        def mk_origin_factory():
            jnode = fs.load_json(args.origin_about_path) or dict()
            o_jnode = jnode.get('origin') or dict()
            l_jnode = jnode.get('location') or dict()

            uuid = o_jnode['uuid']
            ctime = o_jnode['time']
            place = l_jnode.get('place') or ''

            return lambda geolocation: Origin(uuid, ctime, Location(place=place), geolocation)

        self._box_serial = get_hw_serial()
        self._mk_origin = mk_origin_factory()

    def get_origin(self, ts):
        """
        :type ts: long
        :rtype: Origin
        """
        geolocation = self._gpsd.get_geodata(ts)
        return self._mk_origin(geolocation)

    def get_hw_serial(self):
        """
        :rtype: str
        """
        return self._box_serial


def init_service(args, gpsd_service):
    """
    :type args: AboutServiceArgs
    :type gpsd_service: IGpsdService
    :rtype: IAboutService
    """
    return _Service(args, gpsd_service)


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: IAboutService | _Service
    """
    pass
