import datetime

from .types import TzServiceArgs


class ITzService:
    def __init__(self):
        pass

    def get_datetime_isostr(self, ts):
        """
        :type ts: ts
        :rtype: str
        """
        raise NotImplementedError

    def get_datetime(self, ts):
        """
        :type ts: ts
        :rtype: datetime.datetime
        """
        raise NotImplementedError

    def get_datetime_now_isostr(self):
        """
        :rtype: str
        """
        raise NotImplementedError


def init_service(args):
    """
    :type args: TzServiceArgs
    :rtype: ITzService
    """
    return _TzService(args)


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: ITzService | _TzService
    """
    pass


class _TzService(ITzService):
    def __init__(self, args):
        """
        :type args: TzServiceArgs
        """
        ITzService.__init__(self)
        self._tz = args.tz_info

    @staticmethod
    def _dt_to_str(dt):
        """
        :type dt: datetime.datetime
        :rtype: str
        """
        tz_suffix = dt.strftime('%z')
        if tz_suffix == '+0000':
            tz_suffix = 'Z'

        return dt.strftime('%Y-%m-%dT%H:%M:%S') + dt.strftime('.%f')[:4] + tz_suffix

    def get_datetime_isostr(self, ts):
        """
        :param ts: time in seconds since the epoch
        :type ts: float | long | int
        :rtype: str
        """
        # 2018-01-23T15:49:47.124+0400
        dt = self.get_datetime(ts)
        return self._dt_to_str(dt)

    def get_datetime(self, ts):
        """
        :param ts: time in seconds since the epoch
        :type ts: float | long | int
        :rtype: datetime.datetime
        """
        return datetime.datetime.fromtimestamp(float(ts), tz=self._tz)

    def get_datetime_now_isostr(self):
        """
        :rtype: str
        """
        dt = datetime.datetime.now(self._tz)
        return self._dt_to_str(dt)

